
	.import BSOUT
	.importzp ST

	.include "pet.inc"
	.include "peskytone_struct.s"
   
CRTC_REG=$e880
CRTC_VAL=$e881

CHROUT  = $ffd2		; character output (A=code)
GETIN   = $ffe4		; get input (blocking)
CR      = $0d		; PETSCII code for Carriage Return 

	.ifndef COL80
SCREEN	= $8000		; screen memory
SCR_WDT = 40		; screen width in chars
SCR_HGT	= 25		; screen height in chars
	.endif
	.ifdef COL80
SCREEN	= $8014		; screen memory
SCR_WDT = 80		; screen width in chars
SCR_HGT	= 25		; screen height in chars
	.endif
PIC_WDT = 40		; picture width is always the same

; ------------------------------------------------------------------------

.segment "ZEROPAGE"

ptr1_zp:	.res 2
ptr2_zp:	.res 2
cnt1_zp:	.res 1
cnt2_zp:	.res 1
len_zp:		.res 1
spsave:		.res 1


; ------------------------------------------------------------------------
; Startup code

.segment "INIT"
.segment "STARTUP"

Start:

	lda FNLEN
	pha
	lda #14
	jsr BSOUT
	pla
	sta FNLEN

	tsx
	stx spsave

	jsr main
	
	lda #0
	sta ST

	ldx spsave
	txs 
	
	rts

	
	

main:
	
	;clear screen
	
	lda #$93
	sta rand_seed1
	jsr CHROUT
	
	;turn on 'graphics' mode (removes gaps between character rows)
	
	lda #142
	jsr CHROUT
	
	;fill logo back buf

	ldx #0
	
fill_buf:

	jsr rand8
	and #15
	tay
	lda logo_chars,y
	sta logo_back,x
	inx
	bne fill_buf
	
		
	lda #0
	sta cnt1_zp

show_loop:

	jsr rand8
	and #$f0
	sta VIA_SR
	lda #$10
	sta VIA_CR
	jsr rand8
	and #$f0
	eor cnt1_zp
	sta VIA_T2CL
	
	jsr wait_frame

	lda #<title_screen
	sta ptr1_zp+0
	lda #>title_screen
	sta ptr1_zp+1
	lda #<SCREEN
	sta ptr2_zp+0
	lda #>SCREEN
	sta ptr2_zp+1
	
	ldx cnt1_zp
	lda title_seq,x
	sta cnt2_zp

	ldx #SCR_HGT
@l1:
	ldy #0
@l2:
	lda (ptr1_zp),y
	cmp cnt2_zp
	beq :+
	bcs :++
:
	sta (ptr2_zp),y
:
	iny
	cpy #PIC_WDT
	bne @l2
	
	lda ptr1_zp+0
	clc
	adc #PIC_WDT
	sta ptr1_zp+0
	bcc :+
	inc ptr1_zp+1
:
	lda ptr2_zp+0
	clc
	adc #SCR_WDT
	sta ptr2_zp+0
	bcc :+
	inc ptr2_zp+1
:

	dex
	bne @l1
	
	inc cnt1_zp
	ldx cnt1_zp
	lda title_seq,x
	cmp #255
	bne show_loop
	
	lda #0
	sta VIA_SR	
	
	
	lda #$0c		;keep graphics character set active
	jsr pest_init	;takes A contents to use in subsequent PCR writes
	
	
loop_main:

	jsr wait_frame
	
	lda #31
	jsr anim_update_bulbs
	lda #0
	sta logo_anim
	jsr anim_update_logo
	jsr anim_update_head
	jsr anim_update_sparks

	jsr GETIN
	beq loop_main
		
	cmp #'1'
	bcc :+
	cmp #'8'+1
	bcs :+
	jmp play_music
:

	cmp #$03			;RUN/STOP
	beq exit_program
	cmp #'q'
	beq exit_program
	cmp #'Q'
	beq exit_program
	cmp #'e'
	beq exit_program
	cmp #'E'
	beq exit_program
	cmp #'x'
	beq exit_program
	cmp #'X'
	beq exit_program
	
	jmp loop_main
	
exit_program:

	jsr pest_done
	
	lda #<SCREEN
	sta ptr1_zp+0
	lda #>SCREEN
	sta ptr1_zp+1
	
	lda #SCR_HGT
	sta cnt1_zp
	
clear_loop:

	jsr wait_frame
	
	ldy #0
	lda #' '
:
	sta (ptr1_zp),y
	iny
	cpy #SCR_WDT
	bne :-

	lda ptr1_zp+0
	clc
	adc #SCR_WDT
	sta ptr1_zp+0
	bcc :+
	inc ptr1_zp+1
:

	dec cnt1_zp
	bne clear_loop
	
	jmp $fd16				;BASIC goes slightly nuts (disk catalog corruputed) after exiting, can't figure it out, so doing hard reset

	

	
play_music:

	sec
	sbc #'1'

	pha
	
	asl a
	tax
	lda music_list+0,x
	pha
	lda music_list+1,x
	tax
	pla
	jsr pest_music_load
	jsr pest_music_play
		
	pla
	pha
	jsr invert_line

play_loop:


	lda PV_CHANNELS+CH_WAVE+0*CH_STRUCT_SIZE
	sta anim_left_eye
	sta anim_right_speaker
	
	lda PV_CHANNELS+CH_WAVE+2*CH_STRUCT_SIZE
	sta anim_right_eye
	sta anim_left_speaker
	
	lda PV_CHANNELS+CH_WAVE+1*CH_STRUCT_SIZE
	sta anim_top_speaker
	sta anim_bottom_speaker
	
	lda PV_CHANNELS+CH_WAVE+2*CH_STRUCT_SIZE
	and #15
	sta logo_anim
	
	jsr wait_frame
	
	lda #15
	jsr anim_update_bulbs
	jsr anim_update_logo
	jsr anim_update_head
	jsr anim_update_sparks

	lda PV_MUSIC_STATE
	cmp #MUS_STATE_PLAYING
	bne @done
	
	jsr GETIN
	beq play_loop
	
@done:

	pla
	jsr invert_line
	
	jsr pest_music_stop
	
	jsr anim_reset
	
	jmp loop_main
	
	
	
invert_line:

	tax
	
	lda cur_width,x
	sta len_zp
	
	lda #<(SCREEN+17*SCR_WDT)
	sta ptr1_zp+0
	lda #>(SCREEN+17*SCR_WDT)
	sta ptr1_zp+1
	
	cpx #0
	beq :+++
:
	lda ptr1_zp+0
	clc
	adc #SCR_WDT
	sta ptr1_zp+0
	bcc :+
	inc ptr1_zp+1
:
	dex
	bne :--
:

	ldy #0
:
	lda (ptr1_zp),y
	eor #$80
	sta (ptr1_zp),y
	iny
	cpy len_zp
	bne :-

	rts
	


anim_reset:

	lda #0
	sta anim_left_eye
	sta anim_right_eye
	sta anim_top_speaker
	sta anim_left_speaker
	sta anim_right_speaker
	sta anim_bottom_speaker
	
	tax
:
	sta anim_bulbs,x
	inx
	cpx #7
	bne :-
	
	rts
	
	
	
anim_update_logo:

	lda #<(SCREEN+SCR_WDT*1)
	sta ptr1_zp+0
	lda #>(SCREEN+SCR_WDT*1)
	sta ptr1_zp+1
	
	lda #<(title_screen+PIC_WDT*1)
	sta ptr2_zp+0
	lda #>(title_screen+PIC_WDT*1)
	sta ptr2_zp+1
	
	jsr rand8
	adc logo_off
	sta logo_off
	sta len_zp		;anim buf ptr
	
	lda #14			;logo area height
	sta cnt1_zp
	
@loop1:

	ldy #0
	lda #23			;logo area width
	sta cnt2_zp
	
@loop2:

	lda (ptr2_zp),y
	cmp #$2e
	beq @change
	cmp #$3a
	bne @skip
	
@change:

	ldx logo_anim
	beq @restore
	
	ldx len_zp
	inc len_zp
	lda logo_back,x
	
@restore:

	sta (ptr1_zp),y
	
@skip:

	iny
	dec cnt2_zp
	bne @loop2
	
	lda ptr1_zp+0
	clc
	adc #SCR_WDT
	sta ptr1_zp+0
	bcc :+
	inc ptr1_zp+1
:
	lda ptr2_zp+0
	clc
	adc #PIC_WDT
	sta ptr2_zp+0
	bcc :+
	inc ptr2_zp+1
:

	dec cnt1_zp
	bne @loop1

	rts
	
	
	
anim_update_bulbs:

	inc frame_cnt
	and frame_cnt
	beq :+
	rts
:

	ldx #0
:
	jsr rand8
	and #16
	sta anim_bulbs,x
	inx
	cpx #7
	bne :-

	rts
	
	
	
anim_update_head:

	;left eye
	
	lda #$57
	ldx anim_left_eye
	beq :+
	lda #$51
:
	sta SCREEN+32+11*SCR_WDT

	;right eye
	
	lda #$51
	ldx anim_right_eye
	beq :+
	lda #$57
:	
	sta SCREEN+34+11*SCR_WDT
	
	;top speaker
	
	ldy #$56
	ldx anim_top_speaker
	beq :+
	ldy #$d6
:		
	ldx #0
:
	tya
	sta SCREEN+32+6*SCR_WDT,x
	inx
	cpx #4
	bne :-

	;left speaker

	ldy #$56
	ldx anim_left_speaker
	beq :+
	ldy #$d6
:	
	ldx #0
:
	tya
	sta SCREEN+28+10*SCR_WDT,x
	txa
	clc
	adc #SCR_WDT
	tax
	cpx #SCR_WDT*3
	bne :-
	
	;right speaker
	
	ldy #$56
	ldx anim_right_speaker
	beq :+
	ldy #$d6
:	
	ldx #0
:
	tya
	sta SCREEN+39+10*SCR_WDT,x
	txa
	clc
	adc #SCR_WDT
	tax
	cpx #SCR_WDT*3
	bne :-
	
	;bottom speaker
	
	ldy #$d6
	ldx anim_bottom_speaker
	beq :+
	ldy #$56
:	
	ldx #0
:
	tya
	sta SCREEN+32+14*SCR_WDT,x
	sta SCREEN+32+15*SCR_WDT,x
	inx
	cpx #3
	bne :-
	
	ldx #0
:
	cpx #2
	beq :++
	lda #$d7
	ldy anim_bulbs,x
	beq :+
	lda #$d1
:
	sta SCREEN+30+18*SCR_WDT,x
:
	inx
	cpx #7
	bne :---
	
	rts
	
	
	
anim_update_sparks:

	ldx #0

@update:

	lda spark_add,x
	bpl @add
	
@sub:

	lda spark_ptr,x
	clc
	adc spark_add,x
	bcs :+
	lda spark_add,x
	eor #$ff
	sec
	adc #0
	sta spark_add,x
	lda #0
:
	jmp @next
	
@add:

	lda spark_ptr,x
	clc
	adc spark_add,x
	cmp #18*8
	bcc :+
	lda spark_add,x
	eor #$ff
	sec
	adc #0
	sta spark_add,x
	lda #18*8-1
:

@next:

	sta spark_ptr,x
	
	lsr a
	lsr a
	lsr a
	asl a
	sta spark_off,x
	
	inx
	cpx #2
	bne @update


@draw:

	ldx #0
	ldy #0
	
@drawloop:

	lda sparks_path+0,x
	sta ptr1_zp+0
	lda sparks_path+1,x
	sta ptr1_zp+1
	
	lda #'.'
	cpx spark_off+0
	bne :+
	lda #'*'
:
	cpx #16*2
	bcc :+
	eor #$80
:
	sta (ptr1_zp),y
	
	cpx spark_off+1
	bne :++
	lda #'*'
	cpx #16*2
	bcc :+
	eor #$80
:
	sta (ptr1_zp),y
:
	inx
	inx
	cpx #18*2
	bne @drawloop

	rts
	
	
	
wait_frame:

	lda $99
:
	cmp $99
	beq :-
	
	rts


	
rand1:

	lda rand_seed1
	asl a
	bcc :+
	eor #$cf
:
	sta rand_seed1
	rts

rand2:

	lda rand_seed2
	asl a
	bcc :+
	eor #$d7
:
	sta rand_seed2
	rts

rand8:

	jsr rand1
	jsr rand2
	adc rand_seed1
	rts
	
	
	
title_screen:

	.incbin "title.bin"
	
title_seq:

	.byte $00,$01,$02,$03,$04,$05,$06,$07,$08,$09,$0c,$0d,$0e,$0f,$10,$12,$13,$14,$15,$16
	.byte $17,$19,$20,$27,$2e,$30,$31,$32,$33,$34,$35,$36,$37,$38,$3a,$42,$43,$46,$48,$49
	.byte $4a,$4b,$51,$55,$56,$57,$5f,$61,$62,$66,$69,$7b,$a0,$ae,$af,$c5,$cc,$cd,$ce,$cf
	.byte $d0,$d1,$d6,$d7,$df,$e3,$e4,$e5,$e7,$e9,$fa,$ff
	
logo_chars:	.byte ".:+-=#@!\/$%^&"
			.byte '.'+128
			.byte ':'+128

music_list:

	.word music_data_1
	.word music_data_2
	.word music_data_3
	.word music_data_4
	.word music_data_5
	.word music_data_6
	.word music_data_7
	.word music_data_8
	
music_data_1:	.incbin "music_1.bin"
music_data_2:	.incbin "music_2.bin"
music_data_3:	.incbin "music_3.bin"
music_data_4:	.incbin "music_4.bin"
music_data_5:	.incbin "music_5.bin"
music_data_6:	.incbin "music_6.bin"
music_data_7:	.incbin "music_7.bin"
music_data_8:	.incbin "music_8.bin"

cur_width:

	.byte 18
	.byte 16
	.byte 18
	.byte 14
	.byte 15
	.byte 14
	.byte 22
	.byte 17
	
;18 points

sparks_path:

	.word SCREEN+29+SCR_WDT*7
	.word SCREEN+29+SCR_WDT*6
	.word SCREEN+29+SCR_WDT*5
	.word SCREEN+29+SCR_WDT*4
	.word SCREEN+30+SCR_WDT*4
	.word SCREEN+31+SCR_WDT*4
	.word SCREEN+32+SCR_WDT*4
	.word SCREEN+33+SCR_WDT*4
	.word SCREEN+34+SCR_WDT*4
	.word SCREEN+35+SCR_WDT*4
	.word SCREEN+36+SCR_WDT*4
	.word SCREEN+37+SCR_WDT*4
	.word SCREEN+38+SCR_WDT*4
	.word SCREEN+39+SCR_WDT*4
	.word SCREEN+39+SCR_WDT*5
	.word SCREEN+39+SCR_WDT*6
	.word SCREEN+39+SCR_WDT*7
	.word SCREEN+39+SCR_WDT*8

;5:3, 3 lower bits for fractional speed

spark_ptr:	.byte 0*8,18*8-1
spark_off:	.byte 0,0
spark_add:	.byte 2,3

	.include "peskytone_pet.s"
	.include "peskytone.s"
	
effects_data:

anim_left_eye:			.res 1
anim_right_eye:			.res 1
anim_top_speaker:		.res 1
anim_left_speaker:		.res 1
anim_right_speaker:		.res 1
anim_bottom_speaker:	.res 1
anim_bulbs:				.res 7

rand_seed1:				.res 1
rand_seed2:				.res 1
frame_cnt:				.res 1

logo_off:				.res 1
logo_anim:				.res 1
logo_back:				.res 256

zeropage_save: