MUS_STATE_STOPPED=0
MUS_STATE_PLAYING=1
MUS_STATE_PAUSED=2



.segment "PESKYTONE_CODE"
	
	PEST_NOTE_TABLE

;------------------------------------------------------------------------------
; Initialize sound system
; Clears all variables, init hardware, set up timer IRQ (if required)
; in:  A mask for VIA_PCR on PET (to use required character set)
; out: none
;------------------------------------------------------------------------------

pest_init:

	pha
	
	lda #0
	ldy #PV_VARS_SIZE
:
	dey
	sta PV_VARS,y
	bne :-
	
	pla
	
	PEST_INIT
	
	jsr pest_stop_all
	jsr pest_detect_rate
	
	ldx #14	;NTSC
	tay
	beq :+
	ldx #16	;PAL
:
	stx PV_TEMPO_ADD
	
	rts
	
	
	
;------------------------------------------------------------------------------
; Deinitialize sound system
; Stop sound, restore timer IRQ (if required)
; in:  none
; out: none
;------------------------------------------------------------------------------

pest_done:

	jsr pest_stop_all
	
	PEST_DONE
	
	rts
	
	

;------------------------------------------------------------------------------
; Detect framerate (50/60Hz)
; Used internally for music tempo compensation, can be used externally too
; in:  none
; out: A=0 for 60Hz, !0 for 50Hz
;------------------------------------------------------------------------------
	
pest_detect_rate:

	PEST_DETECT_RATE
	
	rts

	
	
;------------------------------------------------------------------------------
; Load music data, initialize music player vars
; in:  X:A pointer to music data
; out: none
;------------------------------------------------------------------------------

pest_music_load:

	sta PV_SRC+0				;music_loc, used to calculate absolute pointers
	stx PV_SRC+1

	ldy #0
	lda (PV_SRC),y				;copy speed and pattern size
	iny
	sta PV_MUSIC_SPEED
	lda (PV_SRC),y
	iny
	sta PV_PATTERN_LEN

	jsr read_relative_loc 		;make absolute pointer to instruments data
	stx PV_INSTR_LOC+0
	sta PV_INSTR_LOC+1
	
	jsr read_relative_loc 		;make absolute pointer to patterns data
	stx PV_PATTERN_LOC+0
	sta PV_PATTERN_LOC+1
	
	jsr read_relative_loc 		;make absolute pointer to order data
	stx PV_ORDER_START+0
	sta PV_ORDER_START+1
	
	jsr read_relative_loc 		;make absolute pointer to order loop point
	stx PV_ORDER_LOOP+0
	sta PV_ORDER_LOOP+1
	
	;jmp music_stop				;fall through
	
	
	
;------------------------------------------------------------------------------
; Stop playing music
; in:  none
; out: none
;------------------------------------------------------------------------------

pest_music_stop:

	lda PV_SFX_PENDING
	
ps_stop_music_before_mute:

	ldx #MUS_STATE_STOPPED
	stx PV_MUSIC_STATE

ps_mute_music_and_sound:	;A should be $#7f to mute everything

	ldx #31	;note off
	stx PV_CHANNELS+CH_NOTE_BASE+CH_STRUCT_SIZE*0
	stx PV_CHANNELS+CH_NOTE_BASE+CH_STRUCT_SIZE*1
	cmp #$7f
	bne :+
	stx PV_CHANNELS+CH_NOTE_BASE+CH_STRUCT_SIZE*2
:
	rts
	
	
	
;------------------------------------------------------------------------------
; Stop all sounds, including music and sound effect
; in:  none
; out: none
;------------------------------------------------------------------------------

pest_stop_all:

	lda #$7f
	sta PV_SFX_PENDING				;set lowest sfx priority
	bne ps_stop_music_before_mute	;jmp
	
	
	
;------------------------------------------------------------------------------
; Start playing music from beginning
; in:  none
; out: none
;------------------------------------------------------------------------------

pest_music_play:

	jsr pest_music_stop		;make sure music is stopped

	lda PV_MUSIC_SPEED			;force first row update
	sta PV_TEMPO_ACC
	lda PV_PATTERN_LEN			;force first row update
	sta PV_ROW_COUNTER
	
	lda PV_ORDER_START+0
	sta PV_ORDER_LOC+0 
	lda PV_ORDER_START+1
	sta PV_ORDER_LOC+1

	lda #MUS_STATE_PLAYING
	sta PV_MUSIC_STATE
	
	rts
	
	
	
;------------------------------------------------------------------------------
; Pause and resume music
; in:  A=0 unpause, !0 pause
; out: none
;------------------------------------------------------------------------------

pest_music_pause:

	ldx PV_MUSIC_STATE
	beq @skip		;MUS_STATE_STOPPED
	tax
	beq @unpause
@pause:
	lda #MUS_STATE_PAUSED
	sta PV_MUSIC_STATE
	lda PV_SFX_PENDING
	bne ps_mute_music_and_sound	;jmp
@unpause:
	lda #MUS_STATE_PLAYING
	sta PV_MUSIC_STATE
@skip:
	rts
	
	
	
;------------------------------------------------------------------------------	
; Request sound effect to be played.
; Number also defines priority, lower numbers have greater priority.
; I.e. if effect 10 is playing, requesting effect 0 will override it,
; while requesting effect 12 will be ignored.
; in:  A sound effect number, 0..63
; out: none
;------------------------------------------------------------------------------

pest_sound_play:

	sta PV_SRC
	lda PV_SFX_PENDING
	and #$7f
	cmp PV_SRC
	bcc :+
	lda PV_SRC
	ora #$80
	sta PV_SFX_PENDING
:
	rts
	
	

;------------------------------------------------------------------------------	
; Update sound system state and produce sounds
; Should be called 50 or 60 times per second, normally from frame interrupt
; in:  none
; out: none
;------------------------------------------------------------------------------

pest_update:

	lda PV_SFX_PENDING
	bpl pu_no_effect

	and #$7f
	sta PV_SFX_PENDING
	
	ldx #<effects_data
	stx PV_PTR+0
	ldx #>effects_data
	stx PV_PTR+1
	
	ldx #ENV_STRUCT_SIZE*((CHANNELS_ALL-1)*2)
	
	jsr envelopes_set
	
	lda #$2f	;C-4
	sta PV_CHANNELS+CH_NOTE_BASE+CH_STRUCT_SIZE*(CHANNELS_ALL-1)
	
pu_no_effect:

pu_music_player:

	lda PV_MUSIC_STATE
	cmp #MUS_STATE_PLAYING
	beq :+
	jmp pu_env_update
:
	lda PV_TEMPO_ACC
	cmp PV_MUSIC_SPEED
	bcs pu_song_update
	jmp pu_frame_update
	
pu_song_update:

	sec
	sbc PV_MUSIC_SPEED
	sta PV_TEMPO_ACC
	
	lda PV_ROW_COUNTER
	cmp PV_PATTERN_LEN
	bcc pu_row_update
	
pu_order_update:

	ldx #0			;channel struct ptr
	stx PV_ROW_COUNTER
	
@loop:

	ldy #0
;:
	lda PV_ORDER_LOC+0
	sta PV_PTR+0
	lda PV_ORDER_LOC+1
	sta PV_PTR+1
	lda (PV_PTR),y

	cmp #$ff
	bcc :+
	;lda PV_ORDER_LOOP+0		;take loop
	;sta PV_ORDER_LOC+0
	;lda PV_ORDER_LOOP+1
	;sta PV_ORDER_LOC+1
	;bne :-					;jmp, A is always not zero here
	jsr pest_music_stop		;hack for album to just stop playing instead of going loop
	jmp pu_env_update
:
	
	pha
	tya
	sta PV_CHANNELS+CH_SKIP,x	;Y=0
	pla
	
	asl a					;read pattern address from A, up to 128 patterns
	tay
	lda PV_PATTERN_LOC+0
	sta PV_PTR+0
	lda PV_PATTERN_LOC+1
	sta PV_PTR+1
	lda (PV_PTR),y
	clc
	adc PV_PATTERN_LOC+0
	sta PV_CHANNELS+CH_BASE+0,x
	iny
	lda (PV_PTR),y
	adc PV_PATTERN_LOC+1
	sta PV_CHANNELS+CH_BASE+1,x
	
	inc PV_ORDER_LOC+0
	bne :+
	inc PV_ORDER_LOC+1
:

	txa
	clc
	adc #CH_STRUCT_SIZE
	tax
	
	cpx #CH_STRUCT_SIZE*3
	bne @loop

	
pu_row_update:

	ldx #0							;channel struct ptr
	stx PV_CNT
	
@loop:

	lda PV_CHANNELS+CH_SKIP,x
	beq :+
	dec PV_CHANNELS+CH_SKIP,x	
	bne @next
:

	lda PV_CHANNELS+CH_BASE+0,x
	sta PV_PTR+0
	lda PV_CHANNELS+CH_BASE+1,x
	sta PV_PTR+1
	ldy #0
	lda (PV_PTR),y

	inc PV_CHANNELS+CH_BASE+0,x
	bne :+
	inc PV_CHANNELS+CH_BASE+1,x
:

	cmp #31
	bcs :+
	sta PV_CHANNELS+CH_SKIP,x
	bcc @next						;jmp
:
	
	cpx #(CHANNELS_ALL-1)*CH_STRUCT_SIZE	;check if there is active sound effect
	bne :+
	ldy PV_SFX_PENDING
	cpy #$7f
	bne @next
:

	sta PV_CHANNELS+CH_NOTE_BASE,x

	lsr a							;extract instrument number
	lsr a
	lsr a
	lsr a
	lsr a
	
	ldy PV_CNT
	clc
	adc channel_instrument_add,y	;add instrument number depending on channel

	stx PV_TEMP
	ldx channel_envelope_ptr,y		;envelope struct offset
	
	ldy PV_INSTR_LOC+0					;use music instrument data
	sty PV_PTR+0
	ldy PV_INSTR_LOC+1
	sty PV_PTR+1

	jsr envelopes_set

	ldx PV_TEMP
	lda #0
	sta PV_CHANNELS+CH_DETUNE,x
	
@next:

	txa
	clc
	adc #CH_STRUCT_SIZE
	tax
	
	inc PV_CNT
	lda PV_CNT
	cmp #CHANNELS_ALL
	bne @loop
	
	inc PV_ROW_COUNTER
	
	
pu_frame_update:

	lda PV_TEMPO_ACC
	clc
	adc PV_TEMPO_ADD
	sta PV_TEMPO_ACC
	
pu_env_update:

	ldx #0							;env struct ptr

@loop:

	lda PV_ENVS+ENV_BASE+0,x
	sta PV_PTR+0
	lda PV_ENVS+ENV_BASE+1,x
	sta PV_PTR+1
	beq @next
	
	ldy PV_ENVS+ENV_PTR,x

@read:

	lda (PV_PTR),y
	iny
	
	cmp #128						;tag end of envelope
	bne :++
	cpx #(ENVELOPES_ALL-2)*ENV_STRUCT_SIZE	;check if it is volume envelope of third channel
	bne :+
	lda #$7f
	sta PV_SFX_PENDING				;set lowest sfx number
:
	ldy PV_ENVS+ENV_LOOP,x
	jmp @read
:
	cmp #129						;tag loop point
	bne :+
	tya
	sta PV_ENVS+ENV_LOOP,x
	jmp @read
:
	cmp #130						;tag mode toggle
	bne :+
	lda PV_ENVS+ENV_MODE,x
	eor #ENV_FLAG_PITCH
	sta PV_ENVS+ENV_MODE,x
	jmp @read
:
	sta PV_ENVS+ENV_OUT,x			;a normal signed byte

	tya
	sta PV_ENVS+ENV_PTR,x
	
@next:

	txa
	clc
	adc #ENV_STRUCT_SIZE
	tax
	
	cpx #ENVELOPES_ALL*ENV_STRUCT_SIZE
	bne @loop

	
pu_channel_mixer:

	ldx #0		;envelope struct ptr
	ldy #0		;channel struct ptr
	
@loop:

	lda PV_CHANNELS+CH_NOTE_BASE,y
	cmp #31
	bcc @next
	beq @mute

	lda PV_ENVS+ENV_OUT,x
	sta PV_CHANNELS+CH_WAVE,y
	
	lda PV_ENVS+ENV_MODE+ENV_STRUCT_SIZE,x
	bne @pitch

	lda PV_CHANNELS+CH_NOTE_BASE,y
	and #$1f
	
	cpy #CH_STRUCT_SIZE*1
	bne :+
	clc
	adc #12				;shift second channel octave up
:
	clc
	adc PV_ENVS+ENV_OUT+ENV_STRUCT_SIZE,x
		
	sta PV_CHANNELS+CH_NOTE_OUT,y
	jmp @next

@pitch:

	lda PV_CHANNELS+CH_NOTE_BASE,y
	and #$1f
	sta PV_CHANNELS+CH_NOTE_OUT,y
	
	lda PV_ENVS+ENV_OUT+ENV_STRUCT_SIZE,x
	asl a
	asl a
	asl a
	sta PV_CHANNELS+CH_DETUNE,y
		
	jmp @next
	
@mute:

	lda #0
	sta PV_CHANNELS+CH_WAVE,y
	
@next:

	txa
	clc
	adc #ENV_STRUCT_SIZE*2
	tax
	tya
	clc
	adc #CH_STRUCT_SIZE
	tay
	
	cpy #3*CH_STRUCT_SIZE
	bne @loop
	
pu_platform_output:

	PEST_MIX
	
	rts
	
	
	
	
;read relative pointer from PV_SRC:Y
;out: A:X

read_relative_loc:

	lda (PV_SRC),y
	iny
	clc
	adc PV_SRC+0
	tax
	lda (PV_SRC),y
	iny
	adc PV_SRC+1
	rts
	

;A=instrument number, 0..63
;X=offset to envelope struct, two adjanced PV_ENVS
;PV_PTR=pointer to envelope data, different between music and sound effects

envelopes_set:

	asl a			;*2
	asl a			;*4
	tay
	
	lda #2
	sta PV_LOOP

@loop:

	lda (PV_PTR),y
	iny
	clc
	adc PV_PTR+0
	sta PV_ENVS+ENV_BASE+0,x
	lda (PV_PTR),y
	iny
	adc PV_PTR+1
	sta PV_ENVS+ENV_BASE+1,x
	
	lda #0
	sta PV_ENVS+ENV_PTR,x
	sta PV_ENVS+ENV_LOOP,x
	sta PV_ENVS+ENV_MODE,x
	
	txa
	clc
	adc #ENV_STRUCT_SIZE
	tax

	dec PV_LOOP
	bne @loop
	
	rts
	

	
channel_instrument_add:
	.byte 0,7,14

channel_envelope_ptr:
	.byte ENV_STRUCT_SIZE*0,ENV_STRUCT_SIZE*2,ENV_STRUCT_SIZE*4
