.segment "PESKYTONE_VARS"

PV_VARS:

PV_SFX_PENDING:		.res 1	;byte
PV_MUSIC_STATE:		.res 1	;byte
PV_PATTERN_LEN:		.res 1	;byte
PV_PATTERN_LOC:		.res 2	;word
PV_ORDER_LOC:		.res 2	;word
PV_ORDER_START:		.res 2	;word
PV_ORDER_LOOP:		.res 2	;word
PV_INSTR_LOC:		.res 2	;word
PV_MUSIC_SPEED:		.res 1	;byte
PV_TEMPO_ACC:		.res 1	;byte
PV_TEMPO_ADD:		.res 1	;byte
PV_ROW_COUNTER:		.res 1	;byte

PV_ENVS:			.res ENV_STRUCT_SIZE*ENVELOPES_ALL
PV_CHANNELS:		.res CH_STRUCT_SIZE*CHANNELS_ALL

PV_VARS_END:
PV_VARS_SIZE=PV_VARS_END-PV_VARS

.segment "PESKYTONE_ZP" : zeropage

PV_SRC:				.res 2	;word, this is only used in music_load
PV_PTR:				.res 2	;word, this and all following variables only used in update
PV_LOOP:			.res 1	;byte
PV_CNT:				.res 1	;byte
PV_TEMP:			.res 1	;byte

;PET specific

PV_FRAME_RATE:		.res 1
PV_IRQ_SR:			.res 1
PV_IRQ_SR_COPY:		.res 1
PV_PITCH_PREV:		.res 1
PV_PCR_MASK:		.res 1



	.macro PEST_INIT
	
	ora #$c0
	sta PV_PCR_MASK
	
detect_framerate:

	lda $99
:
	cmp $99
	beq :-
	ldx #0
	lda $99
:
	ldy #$20
:
	dey
	bne :-
	inx
	cmp $99
	beq :--

	stx PV_FRAME_RATE	;~$73 50Hz, ~$62 60Hz

setup_irq_handler:

	sei
	
	jsr get_irq_entry_ptr
	
	ldy #0
	lda (PV_SRC),y
	sta old_irq+1
	iny
	lda (PV_SRC),y
	sta old_irq+2
	lda #>custom_irq_handler
	sta (PV_SRC),y
	lda #<custom_irq_handler
	dey
	sta (PV_SRC),y
		
	lda #$7f			;clear all interrupts
	sta VIA_IER
	lda #$c0			;enable T1 time-out IRQ
	sta VIA_IER
	
	cli

	lda #$ff
	sta <PV_IRQ_SR_COPY
	sta <PV_PITCH_PREV

	.endmacro
	
	
;------------------------------------------------------------------------------
	
	
	.macro PEST_DONE
	
restore_irq_handler:

	sei
	
	lda #$7f			;disable all interrupts
	sta VIA_IFR
	sta VIA_IER			;follow with a 4t delay before cli to avoid some 'ghost interrutps'
	
	jsr get_irq_entry_ptr
	
	lda old_irq+1
	ldy #0
	sta (PV_SRC),y
	lda old_irq+2
	iny
	sta (PV_SRC),y
	
	cli
	
	lda #0				;disable shift register to restore datasette access
	sta VIA_CR
	
	.endmacro
	
	
;------------------------------------------------------------------------------
	
	
	.macro PEST_DETECT_RATE
	
	lda #0
	ldx PV_FRAME_RATE
	cpx #$68
	bcc :+
	txa
:	

	.endmacro
	
	
;------------------------------------------------------------------------------


	.macro PEST_MIX
	
	ldx #(CHANNELS_ALL-1)*CH_STRUCT_SIZE
	
@loop:

	lda PV_CHANNELS+CH_WAVE,x
	bne @output
	
	txa
	sec
	sbc #CH_STRUCT_SIZE
	tax
	bne @loop

@output:

	lda PV_CHANNELS+CH_WAVE,x
	
@set_wave:

	cmp <PV_IRQ_SR_COPY
	beq :+
	sta <PV_IRQ_SR_COPY
	sta <PV_IRQ_SR
	sta VIA_SR
:
	
	lda PV_CHANNELS+CH_NOTE_OUT,x

@set_pitch:

	cmp <PV_PITCH_PREV
	beq @done
	sta <PV_PITCH_PREV

	tay
	lda note_table_H,y
	beq @sr
	
@irq:

	lda note_table_L,y
	asl a
	sta VIA_T1CL
	lda note_table_H,y
	rol a
	sta VIA_T1CH

	lda #$c0			;enable T1 free running interrupts
	sta VIA_CR
	
	bne @done			;jmp
	
@sr:

	lda #$10			;disable T1 free running interrupts, shift out free running at T2 rate
	sta VIA_CR
	lda note_table_L,y
	sta VIA_T2CL
	
	lda <PV_IRQ_SR_COPY		;reload shift register to start shifting, this issue mentioned in pet-io-2.txt
	sta VIA_SR
	
@done:
	
	

	.endmacro
	
	
;------------------------------------------------------------------------------


	.macro PEST_NOTE_TABLE
	
get_irq_entry_ptr:

	ldx #0				;check ROM version to install handler
	lda $91
	and #$f0
	cmp #$e0
	beq :+

	lda #$19			;ROM 1.0
	ldx #$02
	bne :++
:
	lda #$90			;ROM 2.0
	ldx #$00
:
	sta <PV_SRC+0
	stx <PV_SRC+1
	
	rts
	
	
	
PIA1_PORTA=$e810
PIA1_PORTB=$e812

custom_irq_handler:

	;no need to store registers, monitor already saved them all

	lda VIA_IFR			
	bpl @standard

@custom:

	lda VIA_T1CL		;4 clear interrupt
	lda <PV_IRQ_SR		;3
	cmp #$80			;2
	rol a				;2
	sta <PV_IRQ_SR		;3
	
	and #$20			;2	bit 5 actually gets outputted, speed optimization
	ora PV_PCR_MASK		;3
	sta VIA_PCR			;4

	pla					;exit early
	tay
	pla
	tax
	pla
	rti
	
@standard:

	lda PIA1_PORTB      ;this acknowledges frame interrupt, otherwise it'll get invoked once again right after cli or rti
	cli					;now interrupts can be re-enabled, so timer1 interrupt can run while standard handler is working,
						;removing distortions from the sound

	jsr pest_update
	
old_irq:
	jmp 0				;self-modified code to avoid jmp (addr) alignment issues

	
	
note_table_L:
	.byte $70,$30,$f4,$bb,$85,$53,$23,$f6
	.byte $cb,$a3,$7d,$59,$38,$18,$fa,$dd
	.byte $c2,$a9,$91,$7b,$65,$51,$3e,$2c
	.byte $1c,$0c,$fd,$ee,$e1,$d4,$c8,$bd
	.byte $b2,$a8,$9f,$96,$8e,$86,$7e,$77
	.byte $70,$6a,$64,$5e,$59,$54,$4f,$4b
	.byte $47,$43,$3f,$3b,$38,$35,$32,$2f
note_table_H:
	.byte $04,$04,$03,$03,$03,$03,$03,$02
	.byte $02,$02,$02,$02,$02,$02,$01,$01
	.byte $01,$01,$01,$01,$01,$01,$01,$01
	.byte $01,$01,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00
	.byte $00,$00,$00,$00,$00,$00,$00,$00

	
	.endmacro
